from textwrap import wrap
from ansys.dpf.core.mapping_types import map_types_to_python
from ansys.dpf.core.outputs import _Outputs, Output
from ansys.dpf import core


class Input:
    """
    Parameters
    ----------
    spec :

    pin :

    operator :

    count_ellipsis : optional
        The default is ``-1``.
    """

    def __init__(self, spec, pin, operator, count_ellipsis=-1):
        self._spec = spec
        self._operator = operator
        self._pin = pin
        self._count_ellipsis = count_ellipsis
        self._python_expected_types = []
        for cpp_type in self._spec.type_names:
            python_type = map_types_to_python[cpp_type]
            if python_type not in self._python_expected_types:
                self._python_expected_types.append(map_types_to_python[cpp_type])
        if len(self._spec.type_names) == 0:
            self._python_expected_types.append("Any")
        docstr = self.__str__()
        self.name = self._spec.name
        if self._count_ellipsis != -1:
            self.name += str(self._count_ellipsis + 1)
        self._update_doc_str(docstr, self.name)

    def connect(self, inpt):
        """Connect any input (an entity or an operator output) to a specified input pin of this operator.

        Parameters
        ----------
        inpt : str, int, double, Field, FieldsContainer, Scoping, DataSources,
        MeshedRegion, Output, Outputs, Operator
            Input of the operator.
        """
        # always convert ranges to lists
        if isinstance(inpt, range):
            inpt = list(inpt)
        elif isinstance(inpt, core.Operator):
            if hasattr(inpt, "outputs"):
                inpt = inpt.outputs
            else:
                raise ValueError(
                    "The operator to connect in input has no outputs "
                    "available. It cannot be connected."
                )
        elif isinstance(inpt, core.Model):
            inpt = inpt.metadata.data_sources

        input_type_name = type(inpt).__name__
        if not (
            input_type_name in self._python_expected_types
            or ["Outputs", "Output", "Any"]
        ):
            for types in self._python_expected_types:
                print(types, end=" ")
            print("types are expected for", self._spec.name, "pin")
            return

        corresponding_pins = []
        self._operator._find_outputs_corresponding_pins(
            self._python_expected_types, inpt, self._pin, corresponding_pins
        )
        if len(corresponding_pins) > 1:
            err_str = "Pin connection is ambiguous, specify the pin with:\n"
            for pin in corresponding_pins:
                err_str += (
                    "   - operator.inputs."
                    + self._spec.name
                    + "(out_op."
                    + inpt._dict_outputs[pin[1]].name
                    + ")"
                )
            raise ValueError(err_str)

        if len(corresponding_pins) == 0:
            err_str = (
                f"The input operator for the {self._spec.name} pin be "
                "one of the following types:\n"
            )
            err_str += "\n".join(
                [f"- {py_type}" for py_type in self._python_expected_types]
            )
            raise TypeError(err_str)

        from ansys.dpf.core.results import Result

        if isinstance(inpt, _Outputs):
            self._operator.connect(self._pin, inpt._operator, corresponding_pins[0][1])
            self._operator.inputs._connected_inputs[self._pin] = {
                corresponding_pins[0][1]: inpt._operator
            }
        elif isinstance(inpt, Output):
            self._operator.connect(self._pin, inpt._operator, inpt._pin)
            self._operator.inputs._connected_inputs[self._pin] = {inpt._pin: inpt}
        elif isinstance(inpt, Result):
            self._operator.connect(self._pin, inpt(), corresponding_pins[0][1])
            self._operator.inputs._connected_inputs[self._pin] = {
                corresponding_pins[0][1]: inpt()
                }
        else:
            self._operator.connect(self._pin, inpt)
            self._operator.inputs._connected_inputs[self._pin] = inpt

        self.__inc_if_ellipsis()

    def __call__(self, inpt):
        self.connect(inpt)

    def _update_doc_str(self, docstr, class_name):
        """Dynamically update the docstring of this instance by creating a class on the fly.

        Parameters
        ----------
        docstr :

        class_name :
        """
        child_class = type(class_name, (self.__class__,), {"__doc__": docstr})
        self.__class__ = child_class

    def __str__(self):
        docstr = self._spec.name + " : "
        type_info = self._python_expected_types.copy()
        if self._spec.optional:
            type_info += ["optional"]
        docstr += ", ".join(type_info) + "\n"
        if self._spec.document:
            docstr += "\n".join(wrap(self._spec.document.capitalize())) + "\n"
        if self._count_ellipsis >= 0:
            docstr += "is ellipsis\n"
        return docstr

    def __inc_if_ellipsis(self):
        if self._count_ellipsis >= 0:
            self._count_ellipsis += 1
            if isinstance(self._operator.inputs, _Inputs):
                self._operator.inputs._add_input(
                    self._pin + self._count_ellipsis, self._spec, self._count_ellipsis
                )


class _Inputs:
    def __init__(self, dict_inputs, operator):
        self._dict_inputs = dict_inputs
        self._operator = operator
        self._inputs = []
        self._connected_inputs = {}

    def __str__(self):
        docstr = "Available inputs:\n"
        for inp in self._inputs:
            tot_string = inp.__str__()
            input_string = tot_string.split("\n")
            input_string1 = input_string[0]
            line = ["   ", "- ", input_string1]
            docstr += "{:<5}{:<4}{:<20}\n".format(*line)
            for inputstr in input_string:
                if inputstr != input_string1:
                    line = ["   ", "  ", inputstr]
                    docstr += "{:<5}{:<4}{:<20}\n".format(*line)
        return docstr

    def connect(self, inpt):
        """Connect any input (an entity or an operator output) to any input pin of this operator.

        Searches for the input type corresponding to the output.

        Parameters
        ----------
        inpt : str, int, double, Field, FieldsContainer, Scoping,
               DataSources, MeshedRegion, ScopingsContainer, CyclicSupport,
               ..., Output, Outputs, Operator
            Input of the operator.
        """
        corresponding_pins = []
        if isinstance(inpt, core.Operator):
            if hasattr(inpt, "outputs"):
                inpt = inpt.outputs
            else:
                raise ValueError(
                    "The operator to connect in input has no outputs "
                    "available. It cannot be connected."
                )
        elif isinstance(inpt, core.Model):
            inpt = inpt.metadata.data_sources

        input_type_name = type(inpt).__name__
        for input_pin in self._inputs:
            self._operator._find_outputs_corresponding_pins(
                input_pin._python_expected_types,
                inpt,
                input_pin._pin,
                corresponding_pins,
            )
        if len(corresponding_pins) > 1:
            err_str = "Pin connection is ambiguous, specify the pin with:\n"
            for pin in corresponding_pins:
                if isinstance(pin, tuple):
                    pin = pin[0]
                err_str += (
                    "   - operator.inputs." + self._dict_inputs[pin].name + "(input)\n"
                )
            raise ValueError(err_str)

        if len(corresponding_pins) == 0:
            err_str = "The input should have one of the expected types:\n"
            for pin, spec in self._dict_inputs.items():
                for cpp_type in spec.type_names:
                    err_str += f"   - {map_types_to_python[cpp_type]}\n"
            raise TypeError(err_str)

        from ansys.dpf.core.results import Result
        if isinstance(inpt, Output):
            self._operator.connect(corresponding_pins[0], inpt._operator, inpt._pin)
            self._connected_inputs[corresponding_pins[0]] = {inpt._pin: inpt._operator}
        elif isinstance(inpt, _Outputs):
            self._operator.connect(
                corresponding_pins[0][0], inpt._operator, corresponding_pins[0][1]
            )
            self._connected_inputs[corresponding_pins[0][0]] = {
                corresponding_pins[0][1]: inpt._operator
            }
        elif isinstance(inpt, Result):
            self._operator.connect(
                corresponding_pins[0][0], inpt(), corresponding_pins[0][1]
            )
            self._connected_inputs[corresponding_pins[0][0]] = {
                corresponding_pins[0][1]: inpt()
            }
        else:
            self._operator.connect(corresponding_pins[0], inpt)
            self._connected_inputs[corresponding_pins[0]] = inpt

    def _add_input(self, pin, spec, count_ellipsis=-1):
        if spec is not None:
            class_input = Input(spec, pin, self._operator, count_ellipsis)
            class_input.__doc__ = spec.name
            if not hasattr(self, class_input.name):
                setattr(self, class_input.name, class_input)
            else:
                setattr(self, "_" + class_input.name, class_input)
            self._inputs.append(class_input)

    def __call__(self, inpt):
        self.connect(inpt)


# Dynamic class Inputs
class Inputs(_Inputs):
    def __init__(self, dict_inputs, operator):
        super().__init__(dict_inputs, operator)

        # dynamically populate input attributes
        for pin, spec in self._dict_inputs.items():
            if spec.ellipsis:
                self._add_input(pin, spec, 0)
            else:
                self._add_input(pin, spec)



"""
nodal_fraction_fc
=================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "averaging" category
"""

class nodal_fraction_fc(Operator):
    """Transform ElementalNodal fields into Nodal fields. Each nodal value is the fraction between the nodal difference and the nodal average. Result is computed on a given node scoping.

      available inputs:
        - fields_container (FieldsContainer)
        - mesh (MeshedRegion) (optional)
        - scoping (Scoping) (optional)
        - denominator (FieldsContainer) (optional)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.averaging.nodal_fraction_fc()

      >>> # Make input connections
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)
      >>> my_denominator = dpf.FieldsContainer()
      >>> op.inputs.denominator.connect(my_denominator)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.averaging.nodal_fraction_fc(fields_container=my_fields_container,mesh=my_mesh,scoping=my_scoping,denominator=my_denominator)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, fields_container=None, mesh=None, scoping=None, denominator=None, config=None, server=None):
        super().__init__(name="nodal_fraction_fc", config = config, server = server)
        self._inputs = InputsNodalFractionFc(self)
        self._outputs = OutputsNodalFractionFc(self)
        if fields_container !=None:
            self.inputs.fields_container.connect(fields_container)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)
        if scoping !=None:
            self.inputs.scoping.connect(scoping)
        if denominator !=None:
            self.inputs.denominator.connect(denominator)

    @staticmethod
    def _spec():
        spec = Specification(description="""Transform ElementalNodal fields into Nodal fields. Each nodal value is the fraction between the nodal difference and the nodal average. Result is computed on a given node scoping.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=True, document="""the mesh region in this pin is used to perform the averaging, if there is no field's support it is used"""), 
                                 3 : PinSpecification(name = "scoping", type_names=["scoping"], optional=True, document="""average only on these nodes, if it is scoping container, the label must correspond to the one of the fields container"""), 
                                 6 : PinSpecification(name = "denominator", type_names=["fields_container"], optional=True, document="""if a fields container is set in this pin, it is used as the denominator of the fraction instead of elemental_nodal_To_nodal_fc""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "nodal_fraction_fc")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsNodalFractionFc 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsNodalFractionFc 
        """
        return super().outputs


#internal name: nodal_fraction_fc
#scripting name: nodal_fraction_fc
class InputsNodalFractionFc(_Inputs):
    """Intermediate class used to connect user inputs to nodal_fraction_fc operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.averaging.nodal_fraction_fc()
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)
      >>> my_denominator = dpf.FieldsContainer()
      >>> op.inputs.denominator.connect(my_denominator)
    """
    def __init__(self, op: Operator):
        super().__init__(nodal_fraction_fc._spec().inputs, op)
        self._fields_container = Input(nodal_fraction_fc._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields_container)
        self._mesh = Input(nodal_fraction_fc._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh)
        self._scoping = Input(nodal_fraction_fc._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._scoping)
        self._denominator = Input(nodal_fraction_fc._spec().input_pin(6), 6, op, -1) 
        self._inputs.append(self._denominator)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.nodal_fraction_fc()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        - pindoc: the mesh region in this pin is used to perform the averaging, if there is no field's support it is used

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.nodal_fraction_fc()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

    @property
    def scoping(self):
        """Allows to connect scoping input to the operator

        - pindoc: average only on these nodes, if it is scoping container, the label must correspond to the one of the fields container

        Parameters
        ----------
        my_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.nodal_fraction_fc()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> #or
        >>> op.inputs.scoping(my_scoping)

        """
        return self._scoping

    @property
    def denominator(self):
        """Allows to connect denominator input to the operator

        - pindoc: if a fields container is set in this pin, it is used as the denominator of the fraction instead of elemental_nodal_To_nodal_fc

        Parameters
        ----------
        my_denominator : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.nodal_fraction_fc()
        >>> op.inputs.denominator.connect(my_denominator)
        >>> #or
        >>> op.inputs.denominator(my_denominator)

        """
        return self._denominator

class OutputsNodalFractionFc(_Outputs):
    """Intermediate class used to get outputs from nodal_fraction_fc operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.averaging.nodal_fraction_fc()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(nodal_fraction_fc._spec().outputs, op)
        self._fields_container = Output(nodal_fraction_fc._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.nodal_fraction_fc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




"""
merge_fields_by_label
=====================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "logic" category
"""

class merge_fields_by_label(Operator):
    """Take a fields container and merge its fields that share the same label value.

      available inputs:
        - fields_container (FieldsContainer)
        - label (str)
        - merged_field_support (AbstractFieldSupport) (optional)
        - sumMerge (bool) (optional)

      available outputs:
        - fields_container (FieldsContainer)
        - merged_field_support (AbstractFieldSupport)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.logic.merge_fields_by_label()

      >>> # Make input connections
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_label = str()
      >>> op.inputs.label.connect(my_label)
      >>> my_merged_field_support = dpf.AbstractFieldSupport()
      >>> op.inputs.merged_field_support.connect(my_merged_field_support)
      >>> my_sumMerge = bool()
      >>> op.inputs.sumMerge.connect(my_sumMerge)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.logic.merge_fields_by_label(fields_container=my_fields_container,label=my_label,merged_field_support=my_merged_field_support,sumMerge=my_sumMerge)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()
      >>> result_merged_field_support = op.outputs.merged_field_support()"""
    def __init__(self, fields_container=None, label=None, merged_field_support=None, sumMerge=None, config=None, server=None):
        super().__init__(name="merge::fields_container_label", config = config, server = server)
        self._inputs = InputsMergeFieldsByLabel(self)
        self._outputs = OutputsMergeFieldsByLabel(self)
        if fields_container !=None:
            self.inputs.fields_container.connect(fields_container)
        if label !=None:
            self.inputs.label.connect(label)
        if merged_field_support !=None:
            self.inputs.merged_field_support.connect(merged_field_support)
        if sumMerge !=None:
            self.inputs.sumMerge.connect(sumMerge)

    @staticmethod
    def _spec():
        spec = Specification(description="""Take a fields container and merge its fields that share the same label value.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "label", type_names=["string"], optional=False, document="""Label identifier that should be merged."""), 
                                 2 : PinSpecification(name = "merged_field_support", type_names=["abstract_field_support"], optional=True, document="""The FieldsContainer's support that has already been merged."""), 
                                 3 : PinSpecification(name = "sumMerge", type_names=["bool"], optional=True, document="""Default is false. If true redundant quantities are summed instead of being ignored.""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "merged_field_support", type_names=["abstract_field_support"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "merge::fields_container_label")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMergeFieldsByLabel 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsMergeFieldsByLabel 
        """
        return super().outputs


#internal name: merge::fields_container_label
#scripting name: merge_fields_by_label
class InputsMergeFieldsByLabel(_Inputs):
    """Intermediate class used to connect user inputs to merge_fields_by_label operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.logic.merge_fields_by_label()
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_label = str()
      >>> op.inputs.label.connect(my_label)
      >>> my_merged_field_support = dpf.AbstractFieldSupport()
      >>> op.inputs.merged_field_support.connect(my_merged_field_support)
      >>> my_sumMerge = bool()
      >>> op.inputs.sumMerge.connect(my_sumMerge)
    """
    def __init__(self, op: Operator):
        super().__init__(merge_fields_by_label._spec().inputs, op)
        self._fields_container = Input(merge_fields_by_label._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields_container)
        self._label = Input(merge_fields_by_label._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._label)
        self._merged_field_support = Input(merge_fields_by_label._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._merged_field_support)
        self._sumMerge = Input(merge_fields_by_label._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._sumMerge)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.merge_fields_by_label()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def label(self):
        """Allows to connect label input to the operator

        - pindoc: Label identifier that should be merged.

        Parameters
        ----------
        my_label : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.merge_fields_by_label()
        >>> op.inputs.label.connect(my_label)
        >>> #or
        >>> op.inputs.label(my_label)

        """
        return self._label

    @property
    def merged_field_support(self):
        """Allows to connect merged_field_support input to the operator

        - pindoc: The FieldsContainer's support that has already been merged.

        Parameters
        ----------
        my_merged_field_support : AbstractFieldSupport, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.merge_fields_by_label()
        >>> op.inputs.merged_field_support.connect(my_merged_field_support)
        >>> #or
        >>> op.inputs.merged_field_support(my_merged_field_support)

        """
        return self._merged_field_support

    @property
    def sumMerge(self):
        """Allows to connect sumMerge input to the operator

        - pindoc: Default is false. If true redundant quantities are summed instead of being ignored.

        Parameters
        ----------
        my_sumMerge : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.merge_fields_by_label()
        >>> op.inputs.sumMerge.connect(my_sumMerge)
        >>> #or
        >>> op.inputs.sumMerge(my_sumMerge)

        """
        return self._sumMerge

class OutputsMergeFieldsByLabel(_Outputs):
    """Intermediate class used to get outputs from merge_fields_by_label operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.logic.merge_fields_by_label()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
      >>> result_merged_field_support = op.outputs.merged_field_support()
    """
    def __init__(self, op: Operator):
        super().__init__(merge_fields_by_label._spec().outputs, op)
        self._fields_container = Output(merge_fields_by_label._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)
        self._merged_field_support = Output(merge_fields_by_label._spec().output_pin(1), 1, op) 
        self._outputs.append(self._merged_field_support)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.merge_fields_by_label()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container

    @property
    def merged_field_support(self):
        """Allows to get merged_field_support output of the operator


        Returns
        ----------
        my_merged_field_support : AbstractFieldSupport, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.merge_fields_by_label()
        >>> # Connect inputs : op.inputs. ...
        >>> result_merged_field_support = op.outputs.merged_field_support() 
        """
        return self._merged_field_support




"""
cms_matrices_provider
=====================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from mapdlOperatorsCore plugin, from "result" category
"""

class cms_matrices_provider(Operator):
    """Read reducted matrices for cms elements. Extract stiffness, damping, mass matrices and load vector from a subfile.

      available inputs:
        - data_sources (DataSources)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.result.cms_matrices_provider()

      >>> # Make input connections
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.result.cms_matrices_provider(data_sources=my_data_sources)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, data_sources=None, config=None, server=None):
        super().__init__(name="cms_matrices_provider", config = config, server = server)
        self._inputs = InputsCmsMatricesProvider(self)
        self._outputs = OutputsCmsMatricesProvider(self)
        if data_sources !=None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec():
        spec = Specification(description="""Read reducted matrices for cms elements. Extract stiffness, damping, mass matrices and load vector from a subfile.""",
                             map_input_pin_spec={
                                 4 : PinSpecification(name = "data_sources", type_names=["data_sources"], optional=False, document="""Data_sources (must contain at list one subfile).""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""Fields container containing in this order : stiffness, damping, mass matrices, and then load vector.""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "cms_matrices_provider")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsCmsMatricesProvider 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsCmsMatricesProvider 
        """
        return super().outputs


#internal name: cms_matrices_provider
#scripting name: cms_matrices_provider
class InputsCmsMatricesProvider(_Inputs):
    """Intermediate class used to connect user inputs to cms_matrices_provider operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.cms_matrices_provider()
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
    """
    def __init__(self, op: Operator):
        super().__init__(cms_matrices_provider._spec().inputs, op)
        self._data_sources = Input(cms_matrices_provider._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._data_sources)

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator

        - pindoc: Data_sources (must contain at list one subfile).

        Parameters
        ----------
        my_data_sources : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cms_matrices_provider()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> #or
        >>> op.inputs.data_sources(my_data_sources)

        """
        return self._data_sources

class OutputsCmsMatricesProvider(_Outputs):
    """Intermediate class used to get outputs from cms_matrices_provider operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.cms_matrices_provider()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(cms_matrices_provider._spec().outputs, op)
        self._fields_container = Output(cms_matrices_provider._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        - pindoc: Fields container containing in this order : stiffness, damping, mass matrices, and then load vector.

        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cms_matrices_provider()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




"""
add
===
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "math" category
"""

class add(Operator):
    """Computes the sum of two fields. If one field's scoping has 'overall' location, then these field's values are applied on the entire other field. if one of the input field is empty, the remaining is forwarded to the output.

      available inputs:
        - fieldA (Field, FieldsContainer)
        - fieldB (Field, FieldsContainer)

      available outputs:
        - field (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.math.add()

      >>> # Make input connections
      >>> my_fieldA = dpf.Field()
      >>> op.inputs.fieldA.connect(my_fieldA)
      >>> my_fieldB = dpf.Field()
      >>> op.inputs.fieldB.connect(my_fieldB)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.math.add(fieldA=my_fieldA,fieldB=my_fieldB)

      >>> # Get output data
      >>> result_field = op.outputs.field()"""
    def __init__(self, fieldA=None, fieldB=None, config=None, server=None):
        super().__init__(name="add", config = config, server = server)
        self._inputs = InputsAdd(self)
        self._outputs = OutputsAdd(self)
        if fieldA !=None:
            self.inputs.fieldA.connect(fieldA)
        if fieldB !=None:
            self.inputs.fieldB.connect(fieldB)

    @staticmethod
    def _spec():
        spec = Specification(description="""Computes the sum of two fields. If one field's scoping has 'overall' location, then these field's values are applied on the entire other field. if one of the input field is empty, the remaining is forwarded to the output.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fieldA", type_names=["field","fields_container"], optional=False, document="""field or fields container with only one field is expected"""), 
                                 1 : PinSpecification(name = "fieldB", type_names=["field","fields_container"], optional=False, document="""field or fields container with only one field is expected""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "add")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsAdd 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsAdd 
        """
        return super().outputs


#internal name: add
#scripting name: add
class InputsAdd(_Inputs):
    """Intermediate class used to connect user inputs to add operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.add()
      >>> my_fieldA = dpf.Field()
      >>> op.inputs.fieldA.connect(my_fieldA)
      >>> my_fieldB = dpf.Field()
      >>> op.inputs.fieldB.connect(my_fieldB)
    """
    def __init__(self, op: Operator):
        super().__init__(add._spec().inputs, op)
        self._fieldA = Input(add._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fieldA)
        self._fieldB = Input(add._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._fieldB)

    @property
    def fieldA(self):
        """Allows to connect fieldA input to the operator

        - pindoc: field or fields container with only one field is expected

        Parameters
        ----------
        my_fieldA : Field, FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.add()
        >>> op.inputs.fieldA.connect(my_fieldA)
        >>> #or
        >>> op.inputs.fieldA(my_fieldA)

        """
        return self._fieldA

    @property
    def fieldB(self):
        """Allows to connect fieldB input to the operator

        - pindoc: field or fields container with only one field is expected

        Parameters
        ----------
        my_fieldB : Field, FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.add()
        >>> op.inputs.fieldB.connect(my_fieldB)
        >>> #or
        >>> op.inputs.fieldB(my_fieldB)

        """
        return self._fieldB

class OutputsAdd(_Outputs):
    """Intermediate class used to get outputs from add operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.add()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field = op.outputs.field()
    """
    def __init__(self, op: Operator):
        super().__init__(add._spec().outputs, op)
        self._field = Output(add._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator


        Returns
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.add()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field() 
        """
        return self._field




"""
split_mesh
==========
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "mesh" category
"""

class split_mesh(Operator):
    """Split the input mesh into several meshes based on a given property (material property be default)

      available inputs:
        - mesh_scoping (Scoping) (optional)
        - mesh (MeshedRegion)
        - property (str)

      available outputs:
        - mesh_controller (MeshesContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.mesh.split_mesh()

      >>> # Make input connections
      >>> my_mesh_scoping = dpf.Scoping()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_property = str()
      >>> op.inputs.property.connect(my_property)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.mesh.split_mesh(mesh_scoping=my_mesh_scoping,mesh=my_mesh,property=my_property)

      >>> # Get output data
      >>> result_mesh_controller = op.outputs.mesh_controller()"""
    def __init__(self, mesh_scoping=None, mesh=None, property=None, config=None, server=None):
        super().__init__(name="split_mesh", config = config, server = server)
        self._inputs = InputsSplitMesh(self)
        self._outputs = OutputsSplitMesh(self)
        if mesh_scoping !=None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)
        if property !=None:
            self.inputs.property.connect(property)

    @staticmethod
    def _spec():
        spec = Specification(description="""Split the input mesh into several meshes based on a given property (material property be default)""",
                             map_input_pin_spec={
                                 1 : PinSpecification(name = "mesh_scoping", type_names=["scoping"], optional=True, document="""Scoping"""), 
                                 7 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=False, document=""""""), 
                                 13 : PinSpecification(name = "property", type_names=["string"], optional=False, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "mesh_controller", type_names=["meshes_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "split_mesh")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsSplitMesh 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsSplitMesh 
        """
        return super().outputs


#internal name: split_mesh
#scripting name: split_mesh
class InputsSplitMesh(_Inputs):
    """Intermediate class used to connect user inputs to split_mesh operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.mesh.split_mesh()
      >>> my_mesh_scoping = dpf.Scoping()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_property = str()
      >>> op.inputs.property.connect(my_property)
    """
    def __init__(self, op: Operator):
        super().__init__(split_mesh._spec().inputs, op)
        self._mesh_scoping = Input(split_mesh._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh_scoping)
        self._mesh = Input(split_mesh._spec().input_pin(7), 7, op, -1) 
        self._inputs.append(self._mesh)
        self._property = Input(split_mesh._spec().input_pin(13), 13, op, -1) 
        self._inputs.append(self._property)

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator

        - pindoc: Scoping

        Parameters
        ----------
        my_mesh_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.split_mesh()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> #or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)

        """
        return self._mesh_scoping

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.split_mesh()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

    @property
    def property(self):
        """Allows to connect property input to the operator

        Parameters
        ----------
        my_property : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.split_mesh()
        >>> op.inputs.property.connect(my_property)
        >>> #or
        >>> op.inputs.property(my_property)

        """
        return self._property

class OutputsSplitMesh(_Outputs):
    """Intermediate class used to get outputs from split_mesh operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.mesh.split_mesh()
      >>> # Connect inputs : op.inputs. ...
      >>> result_mesh_controller = op.outputs.mesh_controller()
    """
    def __init__(self, op: Operator):
        super().__init__(split_mesh._spec().outputs, op)
        self._mesh_controller = Output(split_mesh._spec().output_pin(0), 0, op) 
        self._outputs.append(self._mesh_controller)

    @property
    def mesh_controller(self):
        """Allows to get mesh_controller output of the operator


        Returns
        ----------
        my_mesh_controller : MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.split_mesh()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mesh_controller = op.outputs.mesh_controller() 
        """
        return self._mesh_controller




"""
Common
======
"""
import re
from enum import Enum

import progressbar
from ansys.grpc.dpf import base_pb2, field_definition_pb2


def _camel_to_snake_case(name):
    return re.sub(r"(?<!^)(?=[A-Z])", "_", name).lower()


def _remove_spaces(name):
    out = name.lower()
    out = out.replace(" ", "_")
    out = out.replace("-", "_")
    return out


def _snake_to_camel_case(name):
    return "".join(word.title() for word in name.split("_"))


class _smart_dict_camel(dict):
    def __missing__(self, key):
        return _camel_to_snake_case(key)


class _smart_dict_unit_system(dict):
    def __missing__(self, key):
        return "unknown"


def __write_enum_doc__(enum, intro=None):
    str = ""
    if intro:
        str = intro + " \n\n"
    str += "    Attributes\n" + "    -----------\n"
    for e in enum:
        str += "    " + e.name + " \n\n"
    return str


names = [m.lower() for m in base_pb2.Type.keys()]
names.append("fields_container")
names.append("scopings_container")
names.append("meshes_container")
types = Enum("types", names)
types.__doc__ = __write_enum_doc__(
    types,
    (
        "The ``'types'`` enum contains the available types passed "
        "through operators and workflows to DPF."
    ),
)

names = [(m.lower(), num) for m, num in base_pb2.Nature.items()]
natures = Enum("natures", names)
natures.__doc__ = __write_enum_doc__(
    natures,
    (
        "The ``'natures'`` enum contains the dimensionality types.\n "
        "It can be used to create a field of a given dimensionality."
    ),
)


names = [(m.lower(), num - 1) for m, num in field_definition_pb2.ShellLayers.items()]
shell_layers = Enum("shell_layers", names)
shell_layers.__doc__ = __write_enum_doc__(
    shell_layers,
    (
        "The ``'shell_layers'`` enum contains the available order of "
        "shell layers (or lack of shell layers) that defines how the "
        "field's data is ordered."
    ),
)


class locations:
    """Contains strings for scoping and field locations.

    Attributes
    -----------
    none = "none"

    elemental = "Elemental"
        data is one per element

    elemental_nodal = "ElementalNodal"
        one per node per element

    nodal = "Nodal"
        one per node

    time_freq = "TimeFreq_sets"
        one per time set

    overall = "overall"
        applies everywhere

    time_freq_step = "TimeFreq_steps"
        one per time step
    """

    none = "none"

    # data is one per element
    elemental = "Elemental"

    # one per node per element
    elemental_nodal = "ElementalNodal"

    # one per node
    nodal = "Nodal"

    # one per time set
    time_freq = "TimeFreq_sets"

    # applies everywhere
    overall = "overall"

    # one per time step
    time_freq_step = "TimeFreq_steps"


class DefinitionLabels:
    """Contains Python definition labels."""

    time = "time"
    complex = "complex"


def _common_progress_bar(text, unit, tot_size=None):

    if tot_size:
        widgets = [
            progressbar.FormatLabel(f"{text}: %(value)d of %(max_value)d {unit} "),
            progressbar.Percentage(),
            progressbar.Bar(),
        ]
        return progressbar.ProgressBar(widgets=widgets, max_value=tot_size)
    else:
        widgets = [
            progressbar.FormatLabel(f"{text}: %(value)d {unit}"),
            progressbar.RotatingMarker(),
        ]
        return progressbar.ProgressBar(
            widgets=widgets, max_value=progressbar.UnknownLength
        )



"""
Nodes
=====
"""

import numpy as np


from ansys import dpf
from ansys.dpf.core import field, property_field
from ansys.grpc.dpf import meshed_region_pb2
from ansys.dpf.core.errors import protect_grpc


class Node:
    """Encapsulates all properties of a node in the mesh.

    A node is created from the :class:`ansys.dpf.core.elements` or
    :class:`ansys.dpf.core.meshed_region` class.

    Parameters
    ----------
    mesh : :class:`ansys.dpf.core.meshed_region` class
        Mesh region that the node is contained in.
    nodeid : int
        ID of the node.
    nodeindex : int
        Index of the node.
    coordinates : list
        List of ``[x, y, z]`` coordinates for the node.

    Examples
    --------
    >>> import ansys.dpf.core as dpf
    >>> from ansys.dpf.core import examples
    >>> model = dpf.Model(examples.static_rst)
    >>> nodes = model.metadata.meshed_region.nodes

    >>> # Initialize a node from a nodes collection
    >>> node = nodes[0]

    Initialize a node from an element.

    >>> element = model.metadata.meshed_region.elements[0]
    >>> node = element.nodes[0]

    """

    def __init__(self, mesh, nodeid, index, coordinates):
        self._id = nodeid
        self._index = index
        self._coordinates = coordinates
        self._mesh = mesh

    @property
    def index(self) -> int:
        """Index of the node. The index starts at 0."""
        return self._index

    @property
    def id(self) -> int:
        """ID of the node."""
        return self._id

    @property
    def coordinates(self):
        """Cartesian coordinates of the node.

        Examples
        --------
        >>> import ansys.dpf.core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.static_rst)
        >>> node = model.metadata.meshed_region.nodes[0]
        >>> node.coordinates
        [0.015, 0.045, 0.015]

        """
        return self._coordinates

    @property
    def nodal_connectivity(self):
        """Elements indices connected to the node.

        Returns
        -------
        nodal_connectivity : numpy.array
        """
        return self._mesh.nodes.nodal_connectivity_field.get_entity_data(self.index)

    def __str__(self):
        txt = "DPF Node     %7d\n" % self.id
        txt += "Index:      %7d\n" % self.index
        txt += f"Location: {self.coordinates}\n"
        return txt


class Nodes:
    """Provides a collection of DPF nodes.

    Parameters
    ----------
    mesh : :class:`ansys.dpf.core.meshed_region` class
       Mesh region that the collection is created from.

    Examples
    --------
    >>> import ansys.dpf.core as dpf
    >>> from ansys.dpf.core import examples
    >>> model = dpf.Model(examples.static_rst)
    >>> meshed_region = model.metadata.meshed_region
    >>> nodes = model.metadata.meshed_region.nodes
    >>> nodes.n_nodes
    81

    """

    def __init__(self, mesh):
        self._mesh = mesh
        self._mapping_id_to_index = None

    def __str__(self):
        return f"DPF Node collection with {len(self)} nodes\n"

    def __getitem__(self, index):
        """Returns node based on index"""
        return self.node_by_index(index)

    def __len__(self):
        return self.n_nodes

    def __iter__(self):
        for i in range(len(self)):
            yield self[i]

    def node_by_id(self, id):
        """Array of node coordinates ordered by ID."""
        return self.__get_node(nodeid=id)

    def node_by_index(self, index):
        """Array of node coordinates ordered by index"""
        return self.__get_node(nodeindex=index)

    @protect_grpc
    def __get_node(self, nodeindex=None, nodeid=None):
        """Retrieves the node by its ID or index.

        Parameters
        ----------
        nodeid : int
            ID of the node. The default is ``None``.
        nodeindex : int
            Index of the node. The default is ``None``.

        Returns
        -------
        node : ansys.dpf.core.meshed_region.Node
            Requested node
        """
        request = meshed_region_pb2.GetRequest()
        request.mesh.CopyFrom(self._mesh._message)
        if nodeindex is None:
            request.id = nodeid
        else:
            request.index = nodeindex
        nodeOut = self._mesh._stub.GetNode(request)
        return Node(self._mesh, nodeOut.id, nodeOut.index, nodeOut.coordinates)

    @property
    def scoping(self):
        """Scoping of the nodes.

        Returns
        -------
        scoping : Scoping
            Scoping of the nodes.

        Examples
        --------
        Get the IDs of all nodes in this collection.

        >>> import ansys.dpf.core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.static_rst)
        >>> meshed_region = model.metadata.meshed_region
        >>> nodes = model.metadata.meshed_region.nodes
        >>> nodes.scoping.ids[2]
        3

        """
        return self._mesh._get_scoping(loc=dpf.core.locations.nodal)

    @property
    def n_nodes(self):
        """Number of nodes."""
        return self._mesh._stub.List(self._mesh._message).num_nodes

    @property
    def coordinates_field(self):
        """Coordinates field.

        Returns
        -------
        coordinates_field : Field
            Field with all the coordinates for the nodes.

        Examples
        --------
        >>> import ansys.dpf.core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.static_rst)
        >>> meshed_region = model.metadata.meshed_region
        >>> nodes = model.metadata.meshed_region.nodes
        >>> coordinates = nodes.coordinates_field

        >>> # Extract the array of coordinates the coordinates field
        >>> nodes.coordinates_field.data[2]
        array([0.015, 0.045, 0.03 ])

        """
        return self._get_coordinates_field()

    @property
    def nodal_connectivity_field(self):
        """Nodal connectivity field

        Field containing each node ID for the elements indices
        connected to the given node.

        Returns
        -------
        nodal_connectivity_field : PropertyField
            Field of the element indices associated with each node.

        Examples
        --------
        >>> import ansys.dpf.core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.static_rst)
        >>> meshed_region = model.metadata.meshed_region
        >>> nodes = model.metadata.meshed_region.nodes
        >>> field = nodes.nodal_connectivity_field
        >>> field.get_entity_data(1)
        array([0, 2, 4, 6])

        """
        request = meshed_region_pb2.ListPropertyRequest()
        request.mesh.CopyFrom(self._mesh._message)
        request.nodal_property = meshed_region_pb2.NODAL_CONNECTIVITY
        fieldOut = self._mesh._stub.ListProperty(request)
        return property_field.PropertyField(
            server=self._mesh._server, property_field=fieldOut
        )

    @protect_grpc
    def _get_coordinates_field(self):
        """Retrieve the coordinates field."""
        request = meshed_region_pb2.ListPropertyRequest()
        request.mesh.CopyFrom(self._mesh._message)
        # request.nodal_property = meshed_region_pb2.NodalPropertyType.COORDINATES
        request.nodal_property = meshed_region_pb2.COORDINATES
        fieldOut = self._mesh._stub.ListProperty(request)
        return field.Field(server=self._mesh._server, field=fieldOut)

    def _build_mapping_id_to_index(self):
        """Retrieve a mapping between IDs and indices of the entity."""
        return {eid: i for i, eid in enumerate(self.scoping.ids)}

    @property
    def mapping_id_to_index(self):
        if self._mapping_id_to_index is None:
            self._mapping_id_to_index = self._build_mapping_id_to_index()
        return self._mapping_id_to_index

    def map_scoping(self, external_scope):
        """Retrieve the indices to map the scoping of these elements to the scoping of a field.

        Parameters
        ----------
        external_scope : scoping.Scoping
            Scoping to map to.

        Returns
        -------
        indices : numpy.ndarray
            List of indices to map from the external scope to the
            scoping of these nodes.

        mask : numpy.ndarray
            Members of the external scope that are in the node scoping.

        Examples
        --------
        Retrieve the indices that map a field to a nodes collection.

        >>> import ansys.dpf.core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.static_rst)
        >>> nodes = model.metadata.meshed_region.nodes
        >>> disp = model.results.displacement()
        >>> field = disp.outputs.fields_container()[0]
        >>> ind, mask = nodes.map_scoping(field.scoping)

        These indices can then be used to remap the coordinates of the nodes to
        match the order of the field data. This way, the field data matches the
        order of the nodes in the :class:`ansys.dpf.core.meshed_region` class.

        >>> mapped_nodes = nodes.coordinates_field.data[ind]

        """
        if external_scope.location in ["Elemental", "NodalElemental"]:
            raise ValueError('Input scope location must be "Nodal"')
        arr = np.array(list(map(self.mapping_id_to_index.get, external_scope.ids)))
        mask = arr != None
        ind = arr[mask].astype(np.int)
        return ind, mask

    def add_node(self, id, coordinates):
        """Add a node in the mesh.

        Parameters
        ----------
        id : int
            ID for the new node.

        coordinates : list[float]
            List of ``[x, y, z]`` coordinates for the node.
        """
        request = meshed_region_pb2.AddRequest(mesh=self._mesh._message)
        node_request = meshed_region_pb2.NodeRequest(id=id)
        node_request.coordinates.extend(coordinates)
        request.nodes.append(node_request)
        self._mesh._stub.Add(request)

    def add_nodes(self, num):
        """Add a number of nodes in the mesh.

        This method yields a number of nodes that you can define.

        Parameters
        ----------
        num : int
            Number of nodes to add.

        Yields
        ------
        yield node : NodeAdder
            Node to add

        Examples
        --------
        >>> import ansys.dpf.core as dpf
        >>> meshed_region = dpf.MeshedRegion(num_nodes=4,num_elements=3)
        >>> for i, node in enumerate(meshed_region.nodes.add_nodes(4)):
        ...     node.id = i+1
        ...     node.coordinates = [float(i), float(i), 0.0]

        """
        request = meshed_region_pb2.AddRequest(mesh=self._mesh._message)
        for i in range(0, num):
            add = NodeAdder()
            yield add
            node_request = meshed_region_pb2.NodeRequest(id=add.id)
            node_request.coordinates.extend(add.coordinates)
            request.nodes.append(node_request)
        self._mesh._stub.Add(request)


class NodeAdder:
    """Adds a new node to a meshed region.

    Attributes
    ----------
    id : int

    coordinates : list of doubles
        List of ``[x, y, z]'' coordinates.

    Examples
    --------
    Create a meshed region from scratch.

    >>> import ansys.dpf.core as dpf
    >>> meshed_region = dpf.MeshedRegion(num_nodes=4,num_elements=1)
    >>> i=0
    >>> for node in meshed_region.nodes.add_nodes(4):
    ...     node.id = i+1
    ...     node.coordinates = [float(i), float(i), 0.0]
    ...     i=i+1

    """

    def __init__(self):
        self._id = 0
        self._coordinates = [0.0, 0.0, 0.0]

    @property
    def id(self):
        """ID of the node."""
        return self._id

    @id.setter
    def id(self, value):
        self._id = value

    @property
    def coordinates(self):
        """Coordinates of the node."""
        return self._coordinates

    @coordinates.setter
    def coordinates(self, xyz):
        if isinstance(xyz, (np.ndarray, np.generic)):
            self._coordinates = xyz.flatten().tolist()
        else:
            self._coordinates = xyz



"""
run
===
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from mapdlOperatorsCore plugin, from "result" category
"""

class run(Operator):
    """Solve in mapdl a dat/inp file and returns a datasources with the rst file.

      available inputs:
        - mapdl_exe_path (str) (optional)
        - working_dir (str) (optional)
        - number_of_processes (int) (optional)
        - data_sources (DataSources)

      available outputs:
        - data_sources (DataSources)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.result.run()

      >>> # Make input connections
      >>> my_mapdl_exe_path = str()
      >>> op.inputs.mapdl_exe_path.connect(my_mapdl_exe_path)
      >>> my_working_dir = str()
      >>> op.inputs.working_dir.connect(my_working_dir)
      >>> my_number_of_processes = int()
      >>> op.inputs.number_of_processes.connect(my_number_of_processes)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.result.run(mapdl_exe_path=my_mapdl_exe_path,working_dir=my_working_dir,number_of_processes=my_number_of_processes,data_sources=my_data_sources)

      >>> # Get output data
      >>> result_data_sources = op.outputs.data_sources()"""
    def __init__(self, mapdl_exe_path=None, working_dir=None, number_of_processes=None, data_sources=None, config=None, server=None):
        super().__init__(name="mapdl::run", config = config, server = server)
        self._inputs = InputsRun(self)
        self._outputs = OutputsRun(self)
        if mapdl_exe_path !=None:
            self.inputs.mapdl_exe_path.connect(mapdl_exe_path)
        if working_dir !=None:
            self.inputs.working_dir.connect(working_dir)
        if number_of_processes !=None:
            self.inputs.number_of_processes.connect(number_of_processes)
        if data_sources !=None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec():
        spec = Specification(description="""Solve in mapdl a dat/inp file and returns a datasources with the rst file.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "mapdl_exe_path", type_names=["string"], optional=True, document=""""""), 
                                 1 : PinSpecification(name = "working_dir", type_names=["string"], optional=True, document=""""""), 
                                 2 : PinSpecification(name = "number_of_processes", type_names=["int32"], optional=True, document="""Set the number of MPI processes used for resolution (default is 2)"""), 
                                 4 : PinSpecification(name = "data_sources", type_names=["data_sources"], optional=False, document="""data sources containing the input file.""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "data_sources", type_names=["data_sources"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "mapdl::run")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsRun 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsRun 
        """
        return super().outputs


#internal name: mapdl::run
#scripting name: run
class InputsRun(_Inputs):
    """Intermediate class used to connect user inputs to run operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.run()
      >>> my_mapdl_exe_path = str()
      >>> op.inputs.mapdl_exe_path.connect(my_mapdl_exe_path)
      >>> my_working_dir = str()
      >>> op.inputs.working_dir.connect(my_working_dir)
      >>> my_number_of_processes = int()
      >>> op.inputs.number_of_processes.connect(my_number_of_processes)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
    """
    def __init__(self, op: Operator):
        super().__init__(run._spec().inputs, op)
        self._mapdl_exe_path = Input(run._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._mapdl_exe_path)
        self._working_dir = Input(run._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._working_dir)
        self._number_of_processes = Input(run._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._number_of_processes)
        self._data_sources = Input(run._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._data_sources)

    @property
    def mapdl_exe_path(self):
        """Allows to connect mapdl_exe_path input to the operator

        Parameters
        ----------
        my_mapdl_exe_path : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.run()
        >>> op.inputs.mapdl_exe_path.connect(my_mapdl_exe_path)
        >>> #or
        >>> op.inputs.mapdl_exe_path(my_mapdl_exe_path)

        """
        return self._mapdl_exe_path

    @property
    def working_dir(self):
        """Allows to connect working_dir input to the operator

        Parameters
        ----------
        my_working_dir : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.run()
        >>> op.inputs.working_dir.connect(my_working_dir)
        >>> #or
        >>> op.inputs.working_dir(my_working_dir)

        """
        return self._working_dir

    @property
    def number_of_processes(self):
        """Allows to connect number_of_processes input to the operator

        - pindoc: Set the number of MPI processes used for resolution (default is 2)

        Parameters
        ----------
        my_number_of_processes : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.run()
        >>> op.inputs.number_of_processes.connect(my_number_of_processes)
        >>> #or
        >>> op.inputs.number_of_processes(my_number_of_processes)

        """
        return self._number_of_processes

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator

        - pindoc: data sources containing the input file.

        Parameters
        ----------
        my_data_sources : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.run()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> #or
        >>> op.inputs.data_sources(my_data_sources)

        """
        return self._data_sources

class OutputsRun(_Outputs):
    """Intermediate class used to get outputs from run operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.run()
      >>> # Connect inputs : op.inputs. ...
      >>> result_data_sources = op.outputs.data_sources()
    """
    def __init__(self, op: Operator):
        super().__init__(run._spec().outputs, op)
        self._data_sources = Output(run._spec().output_pin(0), 0, op) 
        self._outputs.append(self._data_sources)

    @property
    def data_sources(self):
        """Allows to get data_sources output of the operator


        Returns
        ----------
        my_data_sources : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.run()
        >>> # Connect inputs : op.inputs. ...
        >>> result_data_sources = op.outputs.data_sources() 
        """
        return self._data_sources




"""
Cyclic Support
==============
"""

from ansys import dpf
from ansys.dpf.core.scoping import Scoping
from ansys.grpc.dpf import cyclic_support_pb2, cyclic_support_pb2_grpc


class CyclicSupport:
    """Represents a cyclic support, which describes a model with cyclic symmetry.

    The model has the necessary data for cyclic (and multistage) expansion.

    Parameters
    ----------
    cyclic_support : ansys.grpc.dpf.cyclic_support_pb2.CyclicSupport message
        Cyclic support.
    server : DPFServer , optional
        Server with the channel connected to the remote or local instance. The default is
        ``None``, in which case an attempt is made to use the the global server.

    Examples
    --------
    Get a cyclic support from a model.

    >>> from ansys.dpf import core as dpf
    >>> from ansys.dpf.core import examples
    >>> multi_stage = examples.download_multi_stage_cyclic_result()
    >>> model = dpf.Model(multi_stage)
    >>> result_info = model.metadata.result_info
    >>> cyc_support = result_info.cyclic_support
    >>> cyc_support.num_sectors()
    6
    >>> cyc_support.num_stages
    2

    """

    def __init__(self, cyclic_support, server=None):
        """Initialize time frequency support with its `TimeFreqSupport` message (if possible)."""
        if server is None:
            server = dpf.core._global_server()

        self._server = server
        self._stub = self._connect()
        self._message = cyclic_support

    def __str__(self):
        """Describe the entity.

        Returns
        -------
        str
            Description of the entity.
        """
        from ansys.dpf.core.core import _description

        return _description(self._message, self._server)

    @property
    def num_stages(self) -> int:
        """Number of cyclic stages in the model

        Examples
        --------
        >>> from ansys.dpf.core import Model
        >>> from ansys.dpf.core import examples
        >>> multi_stage = examples.download_multi_stage_cyclic_result()
        >>> cyc_support = Model(multi_stage).metadata.result_info.cyclic_support
        >>> cyc_support.num_stages
        2

        Returns
        -------
        int
            Number of cyclic stages in the model.
        """
        return self._stub.List(self._message).num_stages

    def num_sectors(self, stage_num=0) -> int:
        """Number of sectors to expand on 360 degrees.

        Parameters
        ----------
        stage_num : int , optional
            Number of the stages required (from 0 to num_stages).

        Returns
        -------
        int
            Number of sectors to expand on 360 degrees.

        Examples
        --------
        >>> from ansys.dpf.core import Model
        >>> from ansys.dpf.core import examples
        >>> multi_stage = examples.download_multi_stage_cyclic_result()
        >>> cyc_support = Model(multi_stage).metadata.result_info.cyclic_support
        >>> cyc_support.num_sectors(0)
        6
        >>> cyc_support.num_sectors(1)
        12

        """
        return self._stub.List(self._message).stage_infos[stage_num].num_sectors

    def base_nodes_scoping(self, stage_num=0) -> int:
        """Retrieve a nodal scoping containing node IDs in the
        base sector of the given stage.

        Parameters
        ----------
        stage_num : int, optional
            Number of the stage required (from 0 to num_stages).

        Returns
        -------
        base_nodes_scoping : Scoping
            Nodes IDs in the base sector of the given stage.

        Examples
        --------
        >>> from ansys.dpf.core import Model
        >>> from ansys.dpf.core import examples
        >>> multi_stage = examples.download_multi_stage_cyclic_result()
        >>> cyc_support = Model(multi_stage).metadata.result_info.cyclic_support
        >>> base = cyc_support.base_nodes_scoping(0)

        """
        return Scoping(
            scoping=self._stub.List(self._message)
            .stage_infos[stage_num]
            .base_nodes_scoping,
            server=self._server,
        )

    def base_elements_scoping(self, stage_num=0) -> int:
        """Retrieve an elemental scoping containing elements IDs in the
        base sector of the given stage.

        Parameters
        ----------
        stage_num : int, optional
            Number of the stage required (from 0 to num_stages).

        Returns
        -------
        base_elements_scoping : Scoping
            Elements ids in the base sector of the given stage.

        Examples
        --------
        >>> from ansys.dpf.core import Model
        >>> from ansys.dpf.core import examples
        >>> multi_stage = examples.download_multi_stage_cyclic_result()
        >>> cyc_support = Model(multi_stage).metadata.result_info.cyclic_support
        >>> base = cyc_support.base_elements_scoping(stage_num=1)

        """
        return Scoping(
            scoping=self._stub.List(self._message)
            .stage_infos[stage_num]
            .base_elements_scoping,
            server=self._server,
        )

    def sectors_set_for_expansion(self, stage_num=0) -> int:
        """Retrieve a sector's scoping of the already expanded results
        and mesh or the list of sectors that will be expanded by default.

        A sector's scoping starts from 0, with the maximum equal to num_sectors-1.

        Parameters
        ----------
        stage_num : int, optional
            Number of the stage required (from 0 to num_stages).

        Returns
        -------
        sectors_set_for_expansion : Scoping
            List of sectors (starting from 0 to max = num_sectors-1).

        Examples
        --------
        >>> from ansys.dpf.core import Model
        >>> from ansys.dpf.core import examples
        >>> multi_stage = examples.download_multi_stage_cyclic_result()
        >>> cyc_support = Model(multi_stage).metadata.result_info.cyclic_support
        >>> print(cyc_support.sectors_set_for_expansion(stage_num=1).ids)
        [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]

        """
        return Scoping(
            scoping=self._stub.List(self._message)
            .stage_infos[stage_num]
            .sectors_for_expansion,
            server=self._server,
        )

    def expand_node_id(self, node_id, sectors=None, stage_num=0):
        """Retrieve the node IDs corresponding to the base sector node ID given in the input
        after expansion.

        Parameters
        ----------
        node_id : int
            Base sector's node ID to expand.
        sectors : Scoping , list of int, optional
            List of sectors to expand (from 0 to ``num_sectors - 1``).
            The default is ``None``, in which case all sectors are expanded.
        stage_num : int, optional
            Number of the stage required (from 0 to ``num_stages``).

        Returns
        -------
        sectors_set_for_expansion : Scoping
            List of sectors (starting from 0 to ``num_sectors - 1``).

        Examples
        --------
        >>> from ansys.dpf.core import Model
        >>> from ansys.dpf.core import examples
        >>> multi_stage = examples.download_multi_stage_cyclic_result()
        >>> cyc_support = Model(multi_stage).metadata.result_info.cyclic_support
        >>> print(cyc_support.expand_node_id(1,stage_num=0).ids)
        [1, 3596, 5816, 8036, 10256, 12476]

        """
        if isinstance(sectors, list):
            sectors = Scoping(ids=sectors, location="sectors", server=self._server)

        request = cyclic_support_pb2.GetExpandedIdsRequest()
        request.support.CopyFrom(self._message)
        request.node_id = node_id
        request.stage_num = stage_num
        if sectors:
            request.sectors_to_expand.CopyFrom(sectors._message)
        return Scoping(
            scoping=self._stub.GetExpandedIds(request).expanded_ids, server=self._server
        )

    def expand_element_id(self, element_id, sectors=None, stage_num=0):
        """Retrieves the element IDs corresponding to the base sector element ID given in the input
        after expansion.

        Parameters
        ----------
        element_id : int
            Base sector's element ID to expand.
        sectors : Scoping or list of int, optional
            List of sectors to expand (from 0 to ``num_sectors - 1``).
            The default is ``None``, in which case all sectors are expanded.
        stage_num : int, optional
            Number of the stage required (from 0 to ``num_stages``).

        Returns
        -------
        sectors_set_for_expansion : Scoping
            List of sectors (starting from 0 to ``num_sectors - 1``).

        Examples
        --------
        >>> from ansys.dpf.core import Model
        >>> from ansys.dpf.core import examples
        >>> multi_stage = examples.download_multi_stage_cyclic_result()
        >>> cyc_support = Model(multi_stage).metadata.result_info.cyclic_support
        >>> print(cyc_support.expand_element_id(1,stage_num=0).ids)
        [1, 1558, 2533, 3508, 4483, 5458]

        """
        if isinstance(sectors, list):
            sectors = Scoping(ids=sectors, location="sectors", server=self._server)

        request = cyclic_support_pb2.GetExpandedIdsRequest()
        request.support.CopyFrom(self._message)
        request.element_id = element_id
        request.stage_num = stage_num
        if sectors:
            request.sectors_to_expand.CopyFrom(sectors._message)
        return Scoping(
            scoping=self._stub.GetExpandedIds(request).expanded_ids, server=self._server
        )

    def _connect(self):
        """Connect to the grpc service"""
        return cyclic_support_pb2_grpc.CyclicSupportServiceStub(self._server.channel)

    def __del__(self):
        try:
            self._stub.Delete(self._message)
        except:
            pass



"""
ResultInfo
==========
"""
from enum import Enum

from ansys import dpf
from ansys.grpc.dpf import result_info_pb2, result_info_pb2_grpc
from ansys.dpf.core import available_result
from ansys.dpf.core.mapping_types import map_unit_system
from ansys.dpf.core.cyclic_support import CyclicSupport
from ansys.dpf.core.common import __write_enum_doc__


names = [m for m in result_info_pb2.PhysicsType.keys()]
physics_types = Enum("physics_types", names)
physics_types.__doc__ = __write_enum_doc__(
    physics_types,
    "``'Physics_types'`` enumerates the different types of physics that an analysis can have.",
)

names = [m for m in result_info_pb2.AnalysisType.keys()]
analysis_types = Enum("analysis_types", names)
analysis_types.__doc__ = __write_enum_doc__(
    physics_types, "``'Analysis_types'`` enumerates the different types of analysis."
)


class ResultInfo:
    """Represents the result information.

    This class describes the metadata of the analysis and the available results.

    Parameters
    ----------
    result_info : ansys.grpc.dpf.result_info_pb2.ResultInfo message

     server : ansys.dpf.core.server, optional
        Server with the channel connected to the remote or local instance.
        The default is ``None``, in which case an attempt is made to use the
        global server.

    Examples
    --------
    Explore the result info from the model

    >>> from ansys.dpf import core as dpf
    >>> from ansys.dpf.core import examples
    >>> transient = examples.download_transient_result()
    >>> model = dpf.Model(transient)
    >>> result_info = model.metadata.result_info # printable result_info

    >>> result_info.available_results[0].name
    'displacement'
    >>> result_info.available_results[0].homogeneity
    'length'

    """

    def __init__(self, result_info, server=None):
        """Initialize with a ResultInfo message"""
        if server is None:
            server = dpf.core._global_server()

        self._server = server
        self._stub = self._connect()

        if isinstance(result_info, ResultInfo):
            self._message = result_info._message
        else:
            self._message = result_info

        self._names = [item.name for item in self.available_results]

    def __str__(self):
        txt = (
            "%s analysis\n" % self.analysis_type.capitalize()
            + "Unit system: %s\n" % self.unit_system
            + "Physics Type: %s\n" % self.physics_type.capitalize()
            + "Available results:\n"
        )
        for res in self.available_results:
            line = ["", "-", res.name]
            txt += "{0:^4} {1:^2} {2:<30}".format(*line) + "\n"

        return txt

    def __contains__(self, value):
        return value in self._names

    @property
    def analysis_type(self):
        """Retrieves the analysis type.

        Returns
        -------
        analysis_type : str
            Type of the analysis, such as static or transient.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> result_info = model.metadata.result_info
        >>> result_info.analysis_type
        'static'

        """
        intOut = self._stub.List(self._message).analysis_type
        return result_info_pb2.AnalysisType.Name(intOut).lower()

    @property
    def physics_type(self):
        """Type of the physics.

        Examples
        --------
        Mechanical result

        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> result_info = model.metadata.result_info
        >>> result_info.physics_type
        'mecanic'

        """
        return self._get_physics_type()

    def _get_physics_type(self):
        """
        Returns
        -------
        physics_type : str
            Type of the physics, such as mechanical or electric.
        """
        intOut = self._stub.List(self._message).physics_type
        return result_info_pb2.PhysicsType.Name(intOut).lower()

    # TODO: Depreciate
    @property
    def n_results(self):
        """Number of results."""
        return self._stub.List(self._message).nresult

    @property
    def unit_system(self):
        """Unit system of the result."""
        val = self._stub.List(self._message).unit_system
        return map_unit_system[val]

    @property
    def cyclic_symmetry_type(self):
        """Cyclic symmetry type of the result.

        Return
        ------
        cyclic_symmetry_type : str
            Cyclic symmetry type of the results. Options are ``"single_stage"``,
            ``"multi_stage"``, and ``"not_cyclic"``.
        """
        return self._stub.List(self._message).cyc_info.cyclic_type

    @property
    def has_cyclic(self):
        """Check the result file for cyclic symmetry.

        Return
        ------
        has_cyclic : bool
            Returns ``True`` if the result file has cyclic symmetry or is multistage.
        """
        return self._stub.List(self._message).cyc_info.has_cyclic

    @property
    def cyclic_support(self):
        """Cyclic expansion information for a result file that has cyclic symmetry or is multistage.

        Return
        ------
        cyclic_support : CyclicSupport

        Examples
        --------
        Get a cyclic support from a model.

        >>> from ansys.dpf.core import Model
        >>> from ansys.dpf.core import examples
        >>> multi_stage = examples.download_multi_stage_cyclic_result()
        >>> model = Model(multi_stage)
        >>> result_info = model.metadata.result_info
        >>> cyc_support = result_info.cyclic_support

        """
        tmp = self._stub.List(self._message).cyc_info.cyc_support
        return CyclicSupport(cyclic_support=tmp, server=self._server)

    @property
    def unit_system_name(self):
        """Name of the unit system."""
        return self._stub.List(self._message).unit_system_name

    @property
    def solver_version(self):
        """Version of the solver."""
        major = self._stub.List(self._message).solver_major_version
        minor = self._stub.List(self._message).solver_minor_version
        version = str(major) + "." + str(minor)
        return version

    @property
    def solver_date(self):
        """Date of the solver."""
        return self._stub.List(self._message).solver_date

    @property
    def solver_time(self):
        """Time of the solver."""
        return self._stub.List(self._message).solver_time

    @property
    def user_name(self):
        """Name of the user."""
        return self._stub.List(self._message).user_name

    @property
    def job_name(self):
        """Name of the job."""
        return self._stub.List(self._message).job_name

    @property
    def product_name(self):
        """Name of the product."""
        return self._stub.List(self._message).product_name

    @property
    def main_title(self):
        """Main title."""
        return self._stub.List(self._message).main_title

    @property
    def available_results(self):
        """Available results, containing all information about results
        present in the result files.

        Returns
        -------
        available_result : list[AvailableResult]
        """
        out = []
        for i in range(len(self)):
            out.append(self._get_result(i))
        return out

    def _get_result(self, numres):
        """
        Parameters
        ----------
        numres : int
            Index of the requested result.

        Returns
        -------
        result : Result
        """
        if numres >= len(self):
            raise IndexError("There are only %d results" % len(self))
        elif numres < 0:
            raise IndexError("Result index must be greater than 0")

        request = result_info_pb2.AvailableResultRequest()
        request.result_info.CopyFrom(self._message)
        request.numres = numres
        res = self._stub.ListResult(request)

        return available_result.AvailableResult(res)

    def __len__(self):
        return self._stub.List(self._message).nresult

    def __iter__(self):
        for i in range(len(self)):
            yield self[i]

    def __getitem__(self, key):
        if isinstance(key, int):
            index = key
        elif isinstance(key, str):
            if key not in self._names:
                raise ValueError('Invalid key "%s"' % key)
            index = self._names.index(key)
        else:
            raise TypeError('"%s" is an invalid keytype' % type(key))

        return self._get_result(index)

    def _connect(self):
        """Connect to the gRPC service containing the reader."""
        return result_info_pb2_grpc.ResultInfoServiceStub(self._server.channel)

    def __str__(self):
        """Describe the entity.

        Returns
        -------
        description : str
        """
        from ansys.dpf.core.core import _description

        return _description(self._message, self._server)

    def __del__(self):
        try:
            self._stub.Delete(self._message)
        except:
            pass



import sys
import inspect

## to do : change that one the module is done
from ansys.dpf.core.meshed_region import *  # noqa: F401, F403
from ansys.dpf.core.available_result import *  # noqa: F401, F403
from ansys.dpf.core.data_sources import *  # noqa: F401, F403
from ansys.dpf.core.field import *  # noqa: F401, F403
from ansys.dpf.core.fields_container import *  # noqa: F401, F403
from ansys.dpf.core.scoping import *  # noqa: F401, F403
from ansys.dpf.core.time_freq_support import *  # noqa: F401, F403
from ansys.dpf.core.common import (
    _smart_dict_camel,
    _camel_to_snake_case,
    _snake_to_camel_case,
    _smart_dict_unit_system,
)


map_types_to_cpp = _smart_dict_camel()
for classes in inspect.getmembers(sys.modules[__name__], inspect.isclass):
    map_types_to_cpp[classes[0]] = _camel_to_snake_case(classes[0])
map_types_to_cpp["str"] = "string"
map_types_to_cpp["MeshedRegion"] = "abstract_meshed_region"
map_types_to_cpp["list"] = "vector<int32>"
map_types_to_cpp["bool"] = "bool"
map_types_to_cpp["int"] = "int32"
map_types_to_cpp["double"] = "double"
map_types_to_cpp["float"] = "double"


class _smart_dict_snake(dict):
    def __missing__(self, key):
        return _snake_to_camel_case(key)


map_types_to_python = _smart_dict_snake()
for k, v in map_types_to_cpp.items():
    map_types_to_python[v] = k
map_types_to_python["vector<double>"] = "list"

map_unit_system = _smart_dict_unit_system()
map_unit_system[5] = "Metric (cm, g, dyne, s, V, A)"
map_unit_system[6] = "Metric (mm, Kg, N, s, mV, mA)"
map_unit_system[7] = "U.S Customary (ft, lbm, lbf, F, s, V, A)"
map_unit_system[8] = "U.S Customary (in, lbm, lbf, F, s, V, A)"
map_unit_system[10] = "Metric (um, t, uN, s, V, mA)"
map_unit_system[11] = "Metric (m, kg, N, s, V, A)"



"""
min_over_time_by_entity
=======================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "min_max" category
"""

class min_over_time_by_entity(Operator):
    """Evaluates minimum over time/frequency.

      available inputs:
        - fields_container (FieldsContainer)
        - abs_value (bool) (optional)
        - compute_amplitude (bool) (optional)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.min_max.min_over_time_by_entity()

      >>> # Make input connections
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_abs_value = bool()
      >>> op.inputs.abs_value.connect(my_abs_value)
      >>> my_compute_amplitude = bool()
      >>> op.inputs.compute_amplitude.connect(my_compute_amplitude)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.min_max.min_over_time_by_entity(fields_container=my_fields_container,abs_value=my_abs_value,compute_amplitude=my_compute_amplitude)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, fields_container=None, abs_value=None, compute_amplitude=None, config=None, server=None):
        super().__init__(name="min_over_time_by_entity", config = config, server = server)
        self._inputs = InputsMinOverTimeByEntity(self)
        self._outputs = OutputsMinOverTimeByEntity(self)
        if fields_container !=None:
            self.inputs.fields_container.connect(fields_container)
        if abs_value !=None:
            self.inputs.abs_value.connect(abs_value)
        if compute_amplitude !=None:
            self.inputs.compute_amplitude.connect(compute_amplitude)

    @staticmethod
    def _spec():
        spec = Specification(description="""Evaluates minimum over time/frequency.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document=""""""), 
                                 3 : PinSpecification(name = "abs_value", type_names=["bool"], optional=True, document="""Should use absolute value."""), 
                                 4 : PinSpecification(name = "compute_amplitude", type_names=["bool"], optional=True, document="""Do calculate amplitude.""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "min_over_time_by_entity")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMinOverTimeByEntity 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsMinOverTimeByEntity 
        """
        return super().outputs


#internal name: min_over_time_by_entity
#scripting name: min_over_time_by_entity
class InputsMinOverTimeByEntity(_Inputs):
    """Intermediate class used to connect user inputs to min_over_time_by_entity operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.min_max.min_over_time_by_entity()
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_abs_value = bool()
      >>> op.inputs.abs_value.connect(my_abs_value)
      >>> my_compute_amplitude = bool()
      >>> op.inputs.compute_amplitude.connect(my_compute_amplitude)
    """
    def __init__(self, op: Operator):
        super().__init__(min_over_time_by_entity._spec().inputs, op)
        self._fields_container = Input(min_over_time_by_entity._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields_container)
        self._abs_value = Input(min_over_time_by_entity._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._abs_value)
        self._compute_amplitude = Input(min_over_time_by_entity._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._compute_amplitude)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_over_time_by_entity()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def abs_value(self):
        """Allows to connect abs_value input to the operator

        - pindoc: Should use absolute value.

        Parameters
        ----------
        my_abs_value : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_over_time_by_entity()
        >>> op.inputs.abs_value.connect(my_abs_value)
        >>> #or
        >>> op.inputs.abs_value(my_abs_value)

        """
        return self._abs_value

    @property
    def compute_amplitude(self):
        """Allows to connect compute_amplitude input to the operator

        - pindoc: Do calculate amplitude.

        Parameters
        ----------
        my_compute_amplitude : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_over_time_by_entity()
        >>> op.inputs.compute_amplitude.connect(my_compute_amplitude)
        >>> #or
        >>> op.inputs.compute_amplitude(my_compute_amplitude)

        """
        return self._compute_amplitude

class OutputsMinOverTimeByEntity(_Outputs):
    """Intermediate class used to get outputs from min_over_time_by_entity operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.min_max.min_over_time_by_entity()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(min_over_time_by_entity._spec().outputs, op)
        self._fields_container = Output(min_over_time_by_entity._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_over_time_by_entity()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




"""
centroid_fc
===========
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "math" category
"""

class centroid_fc(Operator):
    """Computes the centroid of all the matching fields of a fields container at a given time or frequency, using fieldOut = field1*(1.-fact)+field2*(fact).

      available inputs:
        - fields_container (FieldsContainer)
        - time_freq (float)
        - step (int) (optional)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.math.centroid_fc()

      >>> # Make input connections
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_time_freq = float()
      >>> op.inputs.time_freq.connect(my_time_freq)
      >>> my_step = int()
      >>> op.inputs.step.connect(my_step)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.math.centroid_fc(fields_container=my_fields_container,time_freq=my_time_freq,step=my_step)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, fields_container=None, time_freq=None, step=None, config=None, server=None):
        super().__init__(name="centroid_fc", config = config, server = server)
        self._inputs = InputsCentroidFc(self)
        self._outputs = OutputsCentroidFc(self)
        if fields_container !=None:
            self.inputs.fields_container.connect(fields_container)
        if time_freq !=None:
            self.inputs.time_freq.connect(time_freq)
        if step !=None:
            self.inputs.step.connect(step)

    @staticmethod
    def _spec():
        spec = Specification(description="""Computes the centroid of all the matching fields of a fields container at a given time or frequency, using fieldOut = field1*(1.-fact)+field2*(fact).""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "time_freq", type_names=["double"], optional=False, document=""""""), 
                                 2 : PinSpecification(name = "step", type_names=["int32"], optional=True, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "centroid_fc")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsCentroidFc 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsCentroidFc 
        """
        return super().outputs


#internal name: centroid_fc
#scripting name: centroid_fc
class InputsCentroidFc(_Inputs):
    """Intermediate class used to connect user inputs to centroid_fc operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.centroid_fc()
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_time_freq = float()
      >>> op.inputs.time_freq.connect(my_time_freq)
      >>> my_step = int()
      >>> op.inputs.step.connect(my_step)
    """
    def __init__(self, op: Operator):
        super().__init__(centroid_fc._spec().inputs, op)
        self._fields_container = Input(centroid_fc._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields_container)
        self._time_freq = Input(centroid_fc._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._time_freq)
        self._step = Input(centroid_fc._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._step)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.centroid_fc()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def time_freq(self):
        """Allows to connect time_freq input to the operator

        Parameters
        ----------
        my_time_freq : float, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.centroid_fc()
        >>> op.inputs.time_freq.connect(my_time_freq)
        >>> #or
        >>> op.inputs.time_freq(my_time_freq)

        """
        return self._time_freq

    @property
    def step(self):
        """Allows to connect step input to the operator

        Parameters
        ----------
        my_step : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.centroid_fc()
        >>> op.inputs.step.connect(my_step)
        >>> #or
        >>> op.inputs.step(my_step)

        """
        return self._step

class OutputsCentroidFc(_Outputs):
    """Intermediate class used to get outputs from centroid_fc operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.centroid_fc()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(centroid_fc._spec().outputs, op)
        self._fields_container = Output(centroid_fc._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.centroid_fc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




"""
vtk_export
==========
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from meshOperatorsCore plugin, from "serialization" category
"""

class vtk_export(Operator):
    """Write the input field and fields container into a given vtk path

      available inputs:
        - file_path (str)
        - mesh (MeshedRegion) (optional)
        - fields1 (FieldsContainer, Field)
        - fields2 (FieldsContainer, Field)

      available outputs:


      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.serialization.vtk_export()

      >>> # Make input connections
      >>> my_file_path = str()
      >>> op.inputs.file_path.connect(my_file_path)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_fields1 = dpf.FieldsContainer()
      >>> op.inputs.fields1.connect(my_fields1)
      >>> my_fields2 = dpf.FieldsContainer()
      >>> op.inputs.fields2.connect(my_fields2)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.serialization.vtk_export(file_path=my_file_path,mesh=my_mesh,fields1=my_fields1,fields2=my_fields2)

      >>> # Get output data"""
    def __init__(self, file_path=None, mesh=None, fields1=None, fields2=None, config=None, server=None):
        super().__init__(name="vtk_export", config = config, server = server)
        self._inputs = InputsVtkExport(self)
        self._outputs = OutputsVtkExport(self)
        if file_path !=None:
            self.inputs.file_path.connect(file_path)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)
        if fields1 !=None:
            self.inputs.fields1.connect(fields1)
        if fields2 !=None:
            self.inputs.fields2.connect(fields2)

    @staticmethod
    def _spec():
        spec = Specification(description="""Write the input field and fields container into a given vtk path""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "file_path", type_names=["string"], optional=False, document="""path with vtk extension were the export occurs"""), 
                                 1 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=True, document="""necessary if the first field or fields container don't have a mesh in their support"""), 
                                 2 : PinSpecification(name = "fields", type_names=["fields_container","field"], optional=False, document="""fields exported"""), 
                                 3 : PinSpecification(name = "fields", type_names=["fields_container","field"], optional=False, document="""fields exported""")},
                             map_output_pin_spec={
})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "vtk_export")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsVtkExport 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsVtkExport 
        """
        return super().outputs


#internal name: vtk_export
#scripting name: vtk_export
class InputsVtkExport(_Inputs):
    """Intermediate class used to connect user inputs to vtk_export operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.serialization.vtk_export()
      >>> my_file_path = str()
      >>> op.inputs.file_path.connect(my_file_path)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_fields1 = dpf.FieldsContainer()
      >>> op.inputs.fields1.connect(my_fields1)
      >>> my_fields2 = dpf.FieldsContainer()
      >>> op.inputs.fields2.connect(my_fields2)
    """
    def __init__(self, op: Operator):
        super().__init__(vtk_export._spec().inputs, op)
        self._file_path = Input(vtk_export._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._file_path)
        self._mesh = Input(vtk_export._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh)
        self._fields1 = Input(vtk_export._spec().input_pin(2), 2, op, 0) 
        self._inputs.append(self._fields1)
        self._fields2 = Input(vtk_export._spec().input_pin(3), 3, op, 1) 
        self._inputs.append(self._fields2)

    @property
    def file_path(self):
        """Allows to connect file_path input to the operator

        - pindoc: path with vtk extension were the export occurs

        Parameters
        ----------
        my_file_path : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.vtk_export()
        >>> op.inputs.file_path.connect(my_file_path)
        >>> #or
        >>> op.inputs.file_path(my_file_path)

        """
        return self._file_path

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        - pindoc: necessary if the first field or fields container don't have a mesh in their support

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.vtk_export()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

    @property
    def fields1(self):
        """Allows to connect fields1 input to the operator

        - pindoc: fields exported

        Parameters
        ----------
        my_fields1 : FieldsContainer, Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.vtk_export()
        >>> op.inputs.fields1.connect(my_fields1)
        >>> #or
        >>> op.inputs.fields1(my_fields1)

        """
        return self._fields1

    @property
    def fields2(self):
        """Allows to connect fields2 input to the operator

        - pindoc: fields exported

        Parameters
        ----------
        my_fields2 : FieldsContainer, Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.vtk_export()
        >>> op.inputs.fields2.connect(my_fields2)
        >>> #or
        >>> op.inputs.fields2(my_fields2)

        """
        return self._fields2

class OutputsVtkExport(_Outputs):
    """Intermediate class used to get outputs from vtk_export operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.serialization.vtk_export()
      >>> # Connect inputs : op.inputs. ...
    """
    def __init__(self, op: Operator):
        super().__init__(vtk_export._spec().outputs, op)
        pass 




"""
cyclic_expanded_acceleration
============================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from mapdlOperatorsCore plugin, from "result" category
"""

class cyclic_expanded_acceleration(Operator):
    """Read acceleration from an rst file and expand it with cyclic symmetry.

      available inputs:
        - time_scoping (Scoping, list) (optional)
        - mesh_scoping (ScopingsContainer, Scoping, list) (optional)
        - fields_container (FieldsContainer) (optional)
        - streams_container (StreamsContainer, Stream) (optional)
        - data_sources (DataSources)
        - bool_rotate_to_global (bool) (optional)
        - sector_mesh (MeshedRegion, MeshesContainer) (optional)
        - requested_location (str) (optional)
        - read_cyclic (int) (optional)
        - expanded_meshed_region (MeshedRegion, MeshesContainer) (optional)
        - cyclic_support (CyclicSupport) (optional)
        - sectors_to_expand (list, Scoping, ScopingsContainer) (optional)
        - phi (float) (optional)

      available outputs:
        - fields_container (FieldsContainer)
        - expanded_meshes (MeshesContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.result.cyclic_expanded_acceleration()

      >>> # Make input connections
      >>> my_time_scoping = dpf.Scoping()
      >>> op.inputs.time_scoping.connect(my_time_scoping)
      >>> my_mesh_scoping = dpf.ScopingsContainer()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
      >>> my_bool_rotate_to_global = bool()
      >>> op.inputs.bool_rotate_to_global.connect(my_bool_rotate_to_global)
      >>> my_sector_mesh = dpf.MeshedRegion()
      >>> op.inputs.sector_mesh.connect(my_sector_mesh)
      >>> my_requested_location = str()
      >>> op.inputs.requested_location.connect(my_requested_location)
      >>> my_read_cyclic = int()
      >>> op.inputs.read_cyclic.connect(my_read_cyclic)
      >>> my_expanded_meshed_region = dpf.MeshedRegion()
      >>> op.inputs.expanded_meshed_region.connect(my_expanded_meshed_region)
      >>> my_cyclic_support = dpf.CyclicSupport()
      >>> op.inputs.cyclic_support.connect(my_cyclic_support)
      >>> my_sectors_to_expand = dpf.list()
      >>> op.inputs.sectors_to_expand.connect(my_sectors_to_expand)
      >>> my_phi = float()
      >>> op.inputs.phi.connect(my_phi)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.result.cyclic_expanded_acceleration(time_scoping=my_time_scoping,mesh_scoping=my_mesh_scoping,fields_container=my_fields_container,streams_container=my_streams_container,data_sources=my_data_sources,bool_rotate_to_global=my_bool_rotate_to_global,sector_mesh=my_sector_mesh,requested_location=my_requested_location,read_cyclic=my_read_cyclic,expanded_meshed_region=my_expanded_meshed_region,cyclic_support=my_cyclic_support,sectors_to_expand=my_sectors_to_expand,phi=my_phi)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()
      >>> result_expanded_meshes = op.outputs.expanded_meshes()"""
    def __init__(self, time_scoping=None, mesh_scoping=None, fields_container=None, streams_container=None, data_sources=None, bool_rotate_to_global=None, sector_mesh=None, requested_location=None, read_cyclic=None, expanded_meshed_region=None, cyclic_support=None, sectors_to_expand=None, phi=None, config=None, server=None):
        super().__init__(name="mapdl::rst::A_cyclic", config = config, server = server)
        self._inputs = InputsCyclicExpandedAcceleration(self)
        self._outputs = OutputsCyclicExpandedAcceleration(self)
        if time_scoping !=None:
            self.inputs.time_scoping.connect(time_scoping)
        if mesh_scoping !=None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if fields_container !=None:
            self.inputs.fields_container.connect(fields_container)
        if streams_container !=None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources !=None:
            self.inputs.data_sources.connect(data_sources)
        if bool_rotate_to_global !=None:
            self.inputs.bool_rotate_to_global.connect(bool_rotate_to_global)
        if sector_mesh !=None:
            self.inputs.sector_mesh.connect(sector_mesh)
        if requested_location !=None:
            self.inputs.requested_location.connect(requested_location)
        if read_cyclic !=None:
            self.inputs.read_cyclic.connect(read_cyclic)
        if expanded_meshed_region !=None:
            self.inputs.expanded_meshed_region.connect(expanded_meshed_region)
        if cyclic_support !=None:
            self.inputs.cyclic_support.connect(cyclic_support)
        if sectors_to_expand !=None:
            self.inputs.sectors_to_expand.connect(sectors_to_expand)
        if phi !=None:
            self.inputs.phi.connect(phi)

    @staticmethod
    def _spec():
        spec = Specification(description="""Read acceleration from an rst file and expand it with cyclic symmetry.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "time_scoping", type_names=["scoping","vector<int32>"], optional=True, document=""""""), 
                                 1 : PinSpecification(name = "mesh_scoping", type_names=["scopings_container","scoping","vector<int32>"], optional=True, document=""""""), 
                                 2 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=True, document="""FieldsContainer already allocated modified inplace"""), 
                                 3 : PinSpecification(name = "streams_container", type_names=["streams_container","stream"], optional=True, document="""Streams containing the result file."""), 
                                 4 : PinSpecification(name = "data_sources", type_names=["data_sources"], optional=False, document="""data sources containing the result file."""), 
                                 5 : PinSpecification(name = "bool_rotate_to_global", type_names=["bool"], optional=True, document="""default is true"""), 
                                 7 : PinSpecification(name = "sector_mesh", type_names=["abstract_meshed_region","meshes_container"], optional=True, document="""mesh of the base sector (can be a skin)."""), 
                                 9 : PinSpecification(name = "requested_location", type_names=["string"], optional=True, document="""location needed in output"""), 
                                 14 : PinSpecification(name = "read_cyclic", type_names=["int32"], optional=True, document="""if 0 cyclic symmetry is ignored, if 1 cyclic sector is read, if 2 cyclic expansion is done, if 3 cyclic expansion is done and stages are merged (default is 1)"""), 
                                 15 : PinSpecification(name = "expanded_meshed_region", type_names=["abstract_meshed_region","meshes_container"], optional=True, document="""mesh expanded."""), 
                                 16 : PinSpecification(name = "cyclic_support", type_names=["cyclic_support"], optional=True, document=""""""), 
                                 18 : PinSpecification(name = "sectors_to_expand", type_names=["vector<int32>","scoping","scopings_container"], optional=True, document="""sectors to expand (start at 0), for multistage: use scopings container with 'stage' label."""), 
                                 19 : PinSpecification(name = "phi", type_names=["double"], optional=True, document="""angle phi (default value 0.0)""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""FieldsContainer filled in"""), 
                                 1 : PinSpecification(name = "expanded_meshes", type_names=["meshes_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "mapdl::rst::A_cyclic")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsCyclicExpandedAcceleration 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsCyclicExpandedAcceleration 
        """
        return super().outputs


#internal name: mapdl::rst::A_cyclic
#scripting name: cyclic_expanded_acceleration
class InputsCyclicExpandedAcceleration(_Inputs):
    """Intermediate class used to connect user inputs to cyclic_expanded_acceleration operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.cyclic_expanded_acceleration()
      >>> my_time_scoping = dpf.Scoping()
      >>> op.inputs.time_scoping.connect(my_time_scoping)
      >>> my_mesh_scoping = dpf.ScopingsContainer()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
      >>> my_bool_rotate_to_global = bool()
      >>> op.inputs.bool_rotate_to_global.connect(my_bool_rotate_to_global)
      >>> my_sector_mesh = dpf.MeshedRegion()
      >>> op.inputs.sector_mesh.connect(my_sector_mesh)
      >>> my_requested_location = str()
      >>> op.inputs.requested_location.connect(my_requested_location)
      >>> my_read_cyclic = int()
      >>> op.inputs.read_cyclic.connect(my_read_cyclic)
      >>> my_expanded_meshed_region = dpf.MeshedRegion()
      >>> op.inputs.expanded_meshed_region.connect(my_expanded_meshed_region)
      >>> my_cyclic_support = dpf.CyclicSupport()
      >>> op.inputs.cyclic_support.connect(my_cyclic_support)
      >>> my_sectors_to_expand = dpf.list()
      >>> op.inputs.sectors_to_expand.connect(my_sectors_to_expand)
      >>> my_phi = float()
      >>> op.inputs.phi.connect(my_phi)
    """
    def __init__(self, op: Operator):
        super().__init__(cyclic_expanded_acceleration._spec().inputs, op)
        self._time_scoping = Input(cyclic_expanded_acceleration._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._time_scoping)
        self._mesh_scoping = Input(cyclic_expanded_acceleration._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh_scoping)
        self._fields_container = Input(cyclic_expanded_acceleration._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._fields_container)
        self._streams_container = Input(cyclic_expanded_acceleration._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._streams_container)
        self._data_sources = Input(cyclic_expanded_acceleration._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._data_sources)
        self._bool_rotate_to_global = Input(cyclic_expanded_acceleration._spec().input_pin(5), 5, op, -1) 
        self._inputs.append(self._bool_rotate_to_global)
        self._sector_mesh = Input(cyclic_expanded_acceleration._spec().input_pin(7), 7, op, -1) 
        self._inputs.append(self._sector_mesh)
        self._requested_location = Input(cyclic_expanded_acceleration._spec().input_pin(9), 9, op, -1) 
        self._inputs.append(self._requested_location)
        self._read_cyclic = Input(cyclic_expanded_acceleration._spec().input_pin(14), 14, op, -1) 
        self._inputs.append(self._read_cyclic)
        self._expanded_meshed_region = Input(cyclic_expanded_acceleration._spec().input_pin(15), 15, op, -1) 
        self._inputs.append(self._expanded_meshed_region)
        self._cyclic_support = Input(cyclic_expanded_acceleration._spec().input_pin(16), 16, op, -1) 
        self._inputs.append(self._cyclic_support)
        self._sectors_to_expand = Input(cyclic_expanded_acceleration._spec().input_pin(18), 18, op, -1) 
        self._inputs.append(self._sectors_to_expand)
        self._phi = Input(cyclic_expanded_acceleration._spec().input_pin(19), 19, op, -1) 
        self._inputs.append(self._phi)

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator

        Parameters
        ----------
        my_time_scoping : Scoping, list, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> #or
        >>> op.inputs.time_scoping(my_time_scoping)

        """
        return self._time_scoping

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator

        Parameters
        ----------
        my_mesh_scoping : ScopingsContainer, Scoping, list, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> #or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)

        """
        return self._mesh_scoping

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        - pindoc: FieldsContainer already allocated modified inplace

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator

        - pindoc: Streams containing the result file.

        Parameters
        ----------
        my_streams_container : StreamsContainer, Stream, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> #or
        >>> op.inputs.streams_container(my_streams_container)

        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator

        - pindoc: data sources containing the result file.

        Parameters
        ----------
        my_data_sources : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> #or
        >>> op.inputs.data_sources(my_data_sources)

        """
        return self._data_sources

    @property
    def bool_rotate_to_global(self):
        """Allows to connect bool_rotate_to_global input to the operator

        - pindoc: default is true

        Parameters
        ----------
        my_bool_rotate_to_global : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> op.inputs.bool_rotate_to_global.connect(my_bool_rotate_to_global)
        >>> #or
        >>> op.inputs.bool_rotate_to_global(my_bool_rotate_to_global)

        """
        return self._bool_rotate_to_global

    @property
    def sector_mesh(self):
        """Allows to connect sector_mesh input to the operator

        - pindoc: mesh of the base sector (can be a skin).

        Parameters
        ----------
        my_sector_mesh : MeshedRegion, MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> op.inputs.sector_mesh.connect(my_sector_mesh)
        >>> #or
        >>> op.inputs.sector_mesh(my_sector_mesh)

        """
        return self._sector_mesh

    @property
    def requested_location(self):
        """Allows to connect requested_location input to the operator

        - pindoc: location needed in output

        Parameters
        ----------
        my_requested_location : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> op.inputs.requested_location.connect(my_requested_location)
        >>> #or
        >>> op.inputs.requested_location(my_requested_location)

        """
        return self._requested_location

    @property
    def read_cyclic(self):
        """Allows to connect read_cyclic input to the operator

        - pindoc: if 0 cyclic symmetry is ignored, if 1 cyclic sector is read, if 2 cyclic expansion is done, if 3 cyclic expansion is done and stages are merged (default is 1)

        Parameters
        ----------
        my_read_cyclic : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> op.inputs.read_cyclic.connect(my_read_cyclic)
        >>> #or
        >>> op.inputs.read_cyclic(my_read_cyclic)

        """
        return self._read_cyclic

    @property
    def expanded_meshed_region(self):
        """Allows to connect expanded_meshed_region input to the operator

        - pindoc: mesh expanded.

        Parameters
        ----------
        my_expanded_meshed_region : MeshedRegion, MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> op.inputs.expanded_meshed_region.connect(my_expanded_meshed_region)
        >>> #or
        >>> op.inputs.expanded_meshed_region(my_expanded_meshed_region)

        """
        return self._expanded_meshed_region

    @property
    def cyclic_support(self):
        """Allows to connect cyclic_support input to the operator

        Parameters
        ----------
        my_cyclic_support : CyclicSupport, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> op.inputs.cyclic_support.connect(my_cyclic_support)
        >>> #or
        >>> op.inputs.cyclic_support(my_cyclic_support)

        """
        return self._cyclic_support

    @property
    def sectors_to_expand(self):
        """Allows to connect sectors_to_expand input to the operator

        - pindoc: sectors to expand (start at 0), for multistage: use scopings container with 'stage' label.

        Parameters
        ----------
        my_sectors_to_expand : list, Scoping, ScopingsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> op.inputs.sectors_to_expand.connect(my_sectors_to_expand)
        >>> #or
        >>> op.inputs.sectors_to_expand(my_sectors_to_expand)

        """
        return self._sectors_to_expand

    @property
    def phi(self):
        """Allows to connect phi input to the operator

        - pindoc: angle phi (default value 0.0)

        Parameters
        ----------
        my_phi : float, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> op.inputs.phi.connect(my_phi)
        >>> #or
        >>> op.inputs.phi(my_phi)

        """
        return self._phi

class OutputsCyclicExpandedAcceleration(_Outputs):
    """Intermediate class used to get outputs from cyclic_expanded_acceleration operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.cyclic_expanded_acceleration()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
      >>> result_expanded_meshes = op.outputs.expanded_meshes()
    """
    def __init__(self, op: Operator):
        super().__init__(cyclic_expanded_acceleration._spec().outputs, op)
        self._fields_container = Output(cyclic_expanded_acceleration._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)
        self._expanded_meshes = Output(cyclic_expanded_acceleration._spec().output_pin(1), 1, op) 
        self._outputs.append(self._expanded_meshes)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        - pindoc: FieldsContainer filled in

        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container

    @property
    def expanded_meshes(self):
        """Allows to get expanded_meshes output of the operator


        Returns
        ----------
        my_expanded_meshes : MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.cyclic_expanded_acceleration()
        >>> # Connect inputs : op.inputs. ...
        >>> result_expanded_meshes = op.outputs.expanded_meshes() 
        """
        return self._expanded_meshes




"""
connectivity_ids
================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "scoping" category
"""

class connectivity_ids(Operator):
    """Returns the ordered node ids corresponding to the element ids scoping in input. For each element the node ids are its connectivity.

      available inputs:
        - mesh_scoping (Scoping)
        - mesh (MeshedRegion) (optional)
        - take_mid_nodes (bool) (optional)

      available outputs:
        - mesh_scoping (Scoping)
        - elemental_scoping (Scoping)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.scoping.connectivity_ids()

      >>> # Make input connections
      >>> my_mesh_scoping = dpf.Scoping()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_take_mid_nodes = bool()
      >>> op.inputs.take_mid_nodes.connect(my_take_mid_nodes)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.scoping.connectivity_ids(mesh_scoping=my_mesh_scoping,mesh=my_mesh,take_mid_nodes=my_take_mid_nodes)

      >>> # Get output data
      >>> result_mesh_scoping = op.outputs.mesh_scoping()
      >>> result_elemental_scoping = op.outputs.elemental_scoping()"""
    def __init__(self, mesh_scoping=None, mesh=None, take_mid_nodes=None, config=None, server=None):
        super().__init__(name="scoping::connectivity_ids", config = config, server = server)
        self._inputs = InputsConnectivityIds(self)
        self._outputs = OutputsConnectivityIds(self)
        if mesh_scoping !=None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)
        if take_mid_nodes !=None:
            self.inputs.take_mid_nodes.connect(take_mid_nodes)

    @staticmethod
    def _spec():
        spec = Specification(description="""Returns the ordered node ids corresponding to the element ids scoping in input. For each element the node ids are its connectivity.""",
                             map_input_pin_spec={
                                 1 : PinSpecification(name = "mesh_scoping", type_names=["scoping"], optional=False, document="""Elemental scoping"""), 
                                 7 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=True, document="""the support of the scoping is expected if there is no mesh in input"""), 
                                 10 : PinSpecification(name = "take_mid_nodes", type_names=["bool"], optional=True, document="""default is true""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "mesh_scoping", type_names=["scoping"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "elemental_scoping", type_names=["scoping"], optional=False, document="""same as the input scoping but with ids dupplicated to havve the same size as nodal output scoping""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "scoping::connectivity_ids")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsConnectivityIds 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsConnectivityIds 
        """
        return super().outputs


#internal name: scoping::connectivity_ids
#scripting name: connectivity_ids
class InputsConnectivityIds(_Inputs):
    """Intermediate class used to connect user inputs to connectivity_ids operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.scoping.connectivity_ids()
      >>> my_mesh_scoping = dpf.Scoping()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_take_mid_nodes = bool()
      >>> op.inputs.take_mid_nodes.connect(my_take_mid_nodes)
    """
    def __init__(self, op: Operator):
        super().__init__(connectivity_ids._spec().inputs, op)
        self._mesh_scoping = Input(connectivity_ids._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh_scoping)
        self._mesh = Input(connectivity_ids._spec().input_pin(7), 7, op, -1) 
        self._inputs.append(self._mesh)
        self._take_mid_nodes = Input(connectivity_ids._spec().input_pin(10), 10, op, -1) 
        self._inputs.append(self._take_mid_nodes)

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator

        - pindoc: Elemental scoping

        Parameters
        ----------
        my_mesh_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.connectivity_ids()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> #or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)

        """
        return self._mesh_scoping

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        - pindoc: the support of the scoping is expected if there is no mesh in input

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.connectivity_ids()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

    @property
    def take_mid_nodes(self):
        """Allows to connect take_mid_nodes input to the operator

        - pindoc: default is true

        Parameters
        ----------
        my_take_mid_nodes : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.connectivity_ids()
        >>> op.inputs.take_mid_nodes.connect(my_take_mid_nodes)
        >>> #or
        >>> op.inputs.take_mid_nodes(my_take_mid_nodes)

        """
        return self._take_mid_nodes

class OutputsConnectivityIds(_Outputs):
    """Intermediate class used to get outputs from connectivity_ids operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.scoping.connectivity_ids()
      >>> # Connect inputs : op.inputs. ...
      >>> result_mesh_scoping = op.outputs.mesh_scoping()
      >>> result_elemental_scoping = op.outputs.elemental_scoping()
    """
    def __init__(self, op: Operator):
        super().__init__(connectivity_ids._spec().outputs, op)
        self._mesh_scoping = Output(connectivity_ids._spec().output_pin(0), 0, op) 
        self._outputs.append(self._mesh_scoping)
        self._elemental_scoping = Output(connectivity_ids._spec().output_pin(1), 1, op) 
        self._outputs.append(self._elemental_scoping)

    @property
    def mesh_scoping(self):
        """Allows to get mesh_scoping output of the operator


        Returns
        ----------
        my_mesh_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.connectivity_ids()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mesh_scoping = op.outputs.mesh_scoping() 
        """
        return self._mesh_scoping

    @property
    def elemental_scoping(self):
        """Allows to get elemental_scoping output of the operator


        - pindoc: same as the input scoping but with ids dupplicated to havve the same size as nodal output scoping

        Returns
        ----------
        my_elemental_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.connectivity_ids()
        >>> # Connect inputs : op.inputs. ...
        >>> result_elemental_scoping = op.outputs.elemental_scoping() 
        """
        return self._elemental_scoping




"""
extract_field
=============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "utility" category
"""

class extract_field(Operator):
    """Extract the fields at the indeces defined in the vector (in 1) form the fields container (in:0).

      available inputs:
        - fields_container (Field, FieldsContainer)
        - indeces (list) (optional)

      available outputs:
        - field (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.utility.extract_field()

      >>> # Make input connections
      >>> my_fields_container = dpf.Field()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_indeces = dpf.list()
      >>> op.inputs.indeces.connect(my_indeces)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.utility.extract_field(fields_container=my_fields_container,indeces=my_indeces)

      >>> # Get output data
      >>> result_field = op.outputs.field()"""
    def __init__(self, fields_container=None, indeces=None, config=None, server=None):
        super().__init__(name="ExtractFromFC", config = config, server = server)
        self._inputs = InputsExtractField(self)
        self._outputs = OutputsExtractField(self)
        if fields_container !=None:
            self.inputs.fields_container.connect(fields_container)
        if indeces !=None:
            self.inputs.indeces.connect(indeces)

    @staticmethod
    def _spec():
        spec = Specification(description="""Extract the fields at the indeces defined in the vector (in 1) form the fields container (in:0).""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["field","fields_container"], optional=False, document="""if a field is in input, it is passed on as output"""), 
                                 1 : PinSpecification(name = "indeces", type_names=["vector<int32>"], optional=True, document="""default is the first field""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "ExtractFromFC")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsExtractField 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsExtractField 
        """
        return super().outputs


#internal name: ExtractFromFC
#scripting name: extract_field
class InputsExtractField(_Inputs):
    """Intermediate class used to connect user inputs to extract_field operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.utility.extract_field()
      >>> my_fields_container = dpf.Field()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_indeces = dpf.list()
      >>> op.inputs.indeces.connect(my_indeces)
    """
    def __init__(self, op: Operator):
        super().__init__(extract_field._spec().inputs, op)
        self._fields_container = Input(extract_field._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields_container)
        self._indeces = Input(extract_field._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._indeces)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        - pindoc: if a field is in input, it is passed on as output

        Parameters
        ----------
        my_fields_container : Field, FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.utility.extract_field()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def indeces(self):
        """Allows to connect indeces input to the operator

        - pindoc: default is the first field

        Parameters
        ----------
        my_indeces : list, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.utility.extract_field()
        >>> op.inputs.indeces.connect(my_indeces)
        >>> #or
        >>> op.inputs.indeces(my_indeces)

        """
        return self._indeces

class OutputsExtractField(_Outputs):
    """Intermediate class used to get outputs from extract_field operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.utility.extract_field()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field = op.outputs.field()
    """
    def __init__(self, op: Operator):
        super().__init__(extract_field._spec().outputs, op)
        self._field = Output(extract_field._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator


        Returns
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.utility.extract_field()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field() 
        """
        return self._field




"""
deserializer
============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "serialization" category
"""

class deserializer(Operator):
    """Takes a file generated by the serializer and deserializes it into DPF's entities.

      available inputs:
        - file_path (str)

      available outputs:
        - any_output1 ()
        - any_output2 ()

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.serialization.deserializer()

      >>> # Make input connections
      >>> my_file_path = str()
      >>> op.inputs.file_path.connect(my_file_path)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.serialization.deserializer(file_path=my_file_path)

      >>> # Get output data
      >>> result_any_output1 = op.outputs.any_output1()
      >>> result_any_output2 = op.outputs.any_output2()"""
    def __init__(self, file_path=None, config=None, server=None):
        super().__init__(name="deserializer", config = config, server = server)
        self._inputs = InputsDeserializer(self)
        self._outputs = OutputsDeserializer(self)
        if file_path !=None:
            self.inputs.file_path.connect(file_path)

    @staticmethod
    def _spec():
        spec = Specification(description="""Takes a file generated by the serializer and deserializes it into DPF's entities.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "file_path", type_names=["string"], optional=False, document="""file path""")},
                             map_output_pin_spec={
                                 1 : PinSpecification(name = "any_output", type_names=[], optional=False, document="""number and types of outputs corresponding of the inputs used in the serialization"""), 
                                 2 : PinSpecification(name = "any_output", type_names=[], optional=False, document="""number and types of outputs corresponding of the inputs used in the serialization""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "deserializer")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsDeserializer 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsDeserializer 
        """
        return super().outputs


#internal name: deserializer
#scripting name: deserializer
class InputsDeserializer(_Inputs):
    """Intermediate class used to connect user inputs to deserializer operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.serialization.deserializer()
      >>> my_file_path = str()
      >>> op.inputs.file_path.connect(my_file_path)
    """
    def __init__(self, op: Operator):
        super().__init__(deserializer._spec().inputs, op)
        self._file_path = Input(deserializer._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._file_path)

    @property
    def file_path(self):
        """Allows to connect file_path input to the operator

        - pindoc: file path

        Parameters
        ----------
        my_file_path : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.deserializer()
        >>> op.inputs.file_path.connect(my_file_path)
        >>> #or
        >>> op.inputs.file_path(my_file_path)

        """
        return self._file_path

class OutputsDeserializer(_Outputs):
    """Intermediate class used to get outputs from deserializer operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.serialization.deserializer()
      >>> # Connect inputs : op.inputs. ...
    """
    def __init__(self, op: Operator):
        super().__init__(deserializer._spec().outputs, op)
        pass 

